﻿namespace VistaDBQuickAndDirtyBrowser
{
	using System;
	using System.Data;
	using System.Diagnostics;
	using System.IO;
	using System.Runtime.InteropServices;
	using System.Windows.Forms;
	using Properties;
	using VistaDB;
	using VistaDB.Provider;

	public partial class MainForm : Form
	{
		public const Int32 WmSyscommand = 0x112;
		public const Int32 MfByposition = 0x400;
		public const Int32 MfSeparator = 0x800;
		public const Int32 CmdInfo = 1000;
		public const Int32 CmdOpenFolder = 1001;

		[DllImport(@"user32.dll")]
		private static extern IntPtr GetSystemMenu(IntPtr hWnd, bool bRevert);
		[DllImport(@"user32.dll")]
		private static extern bool InsertMenu(IntPtr hMenu, Int32 wPosition, Int32 wFlags, Int32 wIDNewItem, string lpNewItem);

		protected override void WndProc(ref Message msg)
		{
			if (msg.Msg == WmSyscommand)
			{
				switch (msg.WParam.ToInt32())
				{
					case CmdInfo:
						Process.Start(@"http://www.zeta-producer.com");
						return;
					case CmdOpenFolder:
						OpenExplorerAndSelectFile(_databaseFilePath);
						return;
					default:
						break;
				}
			}
			base.WndProc(ref msg);
		}

		protected override bool ProcessDialogKey(Keys keyData)
		{
			// http://stackoverflow.com/questions/3300977/arrow-key-events-not-arriving

			if (keyData == Keys.F1)
			{
				Process.Start(@"http://www.zeta-producer.com");
			}
			else if ((keyData & Keys.Control) == Keys.Control && (keyData & Keys.Shift) == Keys.Shift && (keyData & Keys.P) == Keys.P)
			{
				OpenExplorerAndSelectFile(_databaseFilePath);
				return true;
			}

			return base.ProcessDialogKey(keyData);
		}

		private string _databaseFilePath;
		private string _connectionString;
		private bool _ignoreSelectedTableChanged;

		public MainForm()
		{
			InitializeComponent();
		}

		public void Initialize(string databaseFilePath)
		{
			_databaseFilePath = databaseFilePath;
			_connectionString =
				new VistaDBConnectionStringBuilder
					{
						DataSource = databaseFilePath,
						OpenMode = VistaDBDatabaseOpenMode.NonexclusiveReadWrite,
						Pooling = true
					}.ConnectionString;

			// --

			fillTableNames();

			Text += @" - " + Path.GetFileName(_databaseFilePath);
		}

		internal string GetTableColumnType(string tableName, string columnName)
		{
			using (var cn = new VistaDBConnection(_connectionString))
			{
				cn.Open();
				var columns = cn.GetSchema(@"Columns");

				foreach (DataRow row in columns.Rows)
				{
					if (row[@"TABLE_NAME"].ToString() == tableName)
					{
						if (columnName == row[@"COLUMN_NAME"].ToString())
						{
							//return Datatype 
							var x = row[@"DATA_TYPE"];
							return x.ToString();
						}
					}
				}
			}

			throw new Exception();
		}

		private void fillTableNames()
		{
			using (var cn = new VistaDBConnection(_connectionString))
			{
				var selectCommand = new VistaDBCommand
				{
					CommandText = @"select * from [database schema] where typeid = 1 order by [name]",
					Connection = cn
				};

				var da = new VistaDBDataAdapter
							{
								SelectCommand = selectCommand,
								ReturnProviderSpecificTypes = true
							};

				var dataTable = new DataTable();
				da.Fill(dataTable);

				// --

				_ignoreSelectedTableChanged = true;
				tablesListBox.Items.Clear();

				if (dataTable.Rows.Count > 0)
				{
					foreach (DataRow row in dataTable.Rows)
					{
						var name = (string)row[@"name"];
						tablesListBox.Items.Add(name.Trim());
					}

					tablesListBox.SelectedIndex = 0;
				}

				_ignoreSelectedTableChanged = false;
				tablesListBox_SelectedIndexChanged(null, null);
			}
		}

		public static void OpenExplorerAndSelectFile(string filePath)
		{
			// http://stackoverflow.com/questions/334630/open-folder-and-select-the-file
			Process.Start(
				@"explorer.exe",
				string.Format(@"/select, ""{0}""", filePath));
		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			if (Settings.Default.MainWindowWidth > 0) Width = Settings.Default.MainWindowWidth;
			if (Settings.Default.MainWindowHeight > 0) Height = Settings.Default.MainWindowHeight;
			CenterToParent();

			var menuHandle = GetSystemMenu(Handle, false);
			InsertMenu(menuHandle, 5, MfByposition | MfSeparator, 0, string.Empty);
			InsertMenu(menuHandle, 6, MfByposition, CmdInfo, "Info");
			InsertMenu(menuHandle, 7, MfByposition, CmdOpenFolder, "Open folder\tCtrl+Shift+P");

			updateUI();
		}

		private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			Settings.Default.MainWindowWidth = Width;
			Settings.Default.MainWindowHeight = Height;
			Settings.Default.Save();
		}

		private void MainForm_Shown(object sender, EventArgs e)
		{
			BringToFront();
			Activate();
			Focus();
		}

		private void tablesListBox_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (!_ignoreSelectedTableChanged)
			{
				refillGrid();
			}
		}

		private void dataGridView_CellMouseDown(object sender, DataGridViewCellMouseEventArgs e)
		{
			if (e.RowIndex >= 0 && e.ColumnIndex >= 0)
			{
				dataGridView.CurrentCell = dataGridView.Rows[e.RowIndex].Cells[e.ColumnIndex];
			}

			updateUI();
		}

		private void updateUI()
		{
			editToolStripMenuItem.Enabled =
				dataGridView.DataSource != null && dataGridView.CurrentCell != null;

			deleteToolStripMenuItem.Enabled =
				dataGridView.DataSource != null && dataGridView.CurrentCell != null;
		}

		private void dataGridView_CellMouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e)
		{
			updateUI();
			editToolStripMenuItem.PerformClick();
		}

		private void newToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var table = (DataTable)dataGridView.DataSource;

			var row = table.NewRow();
			using (var form = new DetailsForm())
			{
				var added = false;
				form.Initialize(row, tablesListBox.SelectedItem as string, this,
								delegate
								{
									if (!added)
									{
										table.Rows.Add(row);
										added = true;
									}
									writeTableBack(table);

								});

				if (form.ShowDialog(this) == DialogResult.OK)
				{
					refillGrid();

					updateUI();
				}
			}
		}

		private void editToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var row = ((DataRowView)dataGridView.CurrentRow.DataBoundItem).Row;
			using (var form = new DetailsForm())
			{
				form.Initialize(row, tablesListBox.SelectedItem as string, this,
								delegate
								{
									var table = (DataTable)dataGridView.DataSource;
									writeTableBack(table);
								});

				if (form.ShowDialog(this) == DialogResult.OK)
				{
					refillGrid();

					updateUI();
				}
			}
		}

		private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
		{
			if (MessageBox.Show(
				this,
				"Do you really want to delete the selected record?",
				"Confirm delete",
				MessageBoxButtons.YesNo,
				MessageBoxIcon.Question,
				MessageBoxDefaultButton.Button2) == DialogResult.Yes)
			{
				var row = ((DataRowView)dataGridView.CurrentRow.DataBoundItem).Row;
				row.Delete();

				// --

				var table = (DataTable)dataGridView.DataSource;
				writeTableBack(table);
				refillGrid();

				updateUI();
			}
		}

		private void refillGrid()
		{
			using (var cn = new VistaDBConnection(_connectionString))
			{
				var selectCommand =
					new VistaDBCommand
					{
						CommandText = string.Format(@"select * from [{0}]", tablesListBox.SelectedItem),
						Connection = cn
					};

				var da =
					new VistaDBDataAdapter
						{
							SelectCommand = selectCommand,
							ReturnProviderSpecificTypes = true
						};

				var builder = new VistaDBCommandBuilder(da);

				var dataTable = new DataTable();
				da.Fill(dataTable);

				dataGridView.DataSource = dataTable;
			}
		}

		private void writeTableBack(DataTable table)
		{
			using (var cn = new VistaDBConnection(_connectionString))
			{
				var selectCommand =
					new VistaDBCommand
					{
						CommandText = string.Format(@"select * from [{0}]", tablesListBox.SelectedItem),
						Connection = cn
					};

				var da =
					new VistaDBDataAdapter
					{
						SelectCommand = selectCommand
					};

				var builder = new VistaDBCommandBuilder(da);

				//var dataTable = new DataTable();
				//da.Fill(dataTable);

				da.Update(table);

				//dataGridView.DataSource = dataTable;
			}

		}

		private void dataGridView_SelectionChanged(object sender, EventArgs e)
		{
			updateUI();
		}
	}
}